package v1

import (
	"context"
	"go.uber.org/zap"
	"property-management-gateway/pb"
	"property-management-gateway/pb/v1"
	"time"
	"git.getensh.com/common/gopkgs/logger"
)

const (
	ModulePermission = "权限管理"
	ModuleGarden = "小区"
	ModuleCompany = "物业公司"
)
//
const (
	ActionGardenChangeCompany = "小区替换物业"
	ActionCompanyAdd = "新增物业公司"
	ActionPermissionAdd = "增加权限节点"
	ActionPermissionUpdate = "修改权限节点"
	ActionPermissionDel = "删除权限节点"
)



type OperationLogRequest struct {
	Module string
	Action string
	UserName string
	Uid int64
	Origin interface{}
	Target interface{}
}

func OperationLogAdd(req *OperationLogRequest) {
	mreq := v1.ManagementLogAddRequest{
		Module:req.Module,
		Action:req.Action,
		Username:req.UserName,
		Uid:req.Uid,
		Timestamp:time.Now().Unix(),
	}
	if req.Origin != nil {
		origin, _ := json.Marshal(req.Origin)
		mreq.Origin = string(origin)
	}
	if req.Target != nil {
		target, _ := json.Marshal(req.Target)
		mreq.Target = string(target)
	}
	_, err := pb.PropertyLog.ManagementLogAdd(context.Background(), &mreq)
	if err != nil {
		s, _ := json.Marshal(mreq)
		logger.Error("func",
			zap.String("call", "pb.PropertyLog.ManagementLogAdd"),
			zap.String("params", string(s)),
			zap.String("error", err.Error()))
	}
}

